\name{ksline}

\alias{ksline}
\alias{ksline.aux.1}

\title{Performs spatial interpolation (kriging) with fixed covariance
  parameters using global or moving neighborhood}

\description{
  This function performs spatial prediction for fixed covariance
  parameters. It can produce results which corresponds to the following
  kriging types:
  SK (simple kriging), OK (ordinary kriging),
  KTE (external trend kriging), UK (universal kriging).
  
  The function \code{\link{krige.conv}} should be preferred for most
  kriging calculations. An exception is for the case where the moving
  neighborhood is adopted.
}

\usage{
ksline(geodata, coords=geodata$coords, data=geodata$data, locations,
       cov.pars=stop("cov. parameters (sigmasq and phi) needed"),
       nugget=0, micro.scale=0,
       cov.model = c("exponential", "matern", "gaussian",
                   "spherical", "circular", "cubic", "wave",
                   "powered.exponential", "cauchy", "gneiting",
                   "gneiting.matern", "pure.nugget"),
       kappa=0.5, lambda=1, m0="ok", nwin="full",
       n.samples.backtransform=500, trend=1, d=2,
       ktedata=NULL, ktelocations=NULL, aniso.pars=NULL,
       signal=FALSE, dist.epsilon=1e-10, messages.screen=TRUE)
}


\arguments{
  \item{geodata}{a list containing elements \code{$coords} and
    \code{$data} as described next. If not provided the arguments
    \code{coords} and \code{data} must be provided instead.}
  \item{coords}{an \eqn{n \times 2}{n x 2} matrix where each row have the 2-D
    coordinates of the \eqn{n} data locations.
    By default it takes the
    element \code{$coords} of the argument \code{geodata}, if provided.}
  \item{data}{a vector with \emph{n} data values. By default it takes the
    element \code{$data} of the argument \code{geodata}, if provided.}
  \item{locations}{an \eqn{N \times 2}{N x 2} matrix or data-frame with the 2-D
    coordinates of the \eqn{N} locations to be predicted.}
  \item{cov.pars}{a vector with 2 elements or an \eqn{n \times 2}{n x 2} matrix with the covariance parameters
    \eqn{\sigma^2}{sigma^2} (sill)  and \eqn{\phi}{phi} (range). If a
    vetor the sill must be the first element and the range is the second. If a matrix,
    corresponding to a model with several structures, the sill values
    are placed in the first column and the range values in the second column.}  
  \item{nugget}{a number corresponding to the nugget variance.}
  \item{micro.scale}{micro-scale variance. If different from zero, the
    nugget variance is divided in 2 terms: \emph{micro-scale variance}
    and \emph{measurement error}. This might affect the precision of the estimates.
    In practice, these two are usually indistinguishable but it can be
    set here when possible.} 
  \item{cov.model}{model for the correlation function. See options in the
    function call and/or documentation for the function \code{\link{cov.spatial}}.}
  \item{kappa}{additional parameter needed for some of the correlation
    functions: \cr (\code{"matern"}, \code{"powered.exponential"}, \code{"gneiting"} and
    \code{"gneiting.matern"}). For more details see documentation for the
    function \code{\link{cov.spatial}}.}
  \item{lambda}{Box-Cox transformation parameter. If \eqn{\lambda =
      1}{lambda = 1} no transformation is used, otherwise the Box-Cox
    data transformation is performed before the analysis.
    The kriging results are returned in the same scale as the original data.
    If \eqn{\lambda = 0}{lambda = 0} the log-transformation is performed.
    If \eqn{\lambda < 0}{lambda < 0} the mean predictor doesn't make sense
    (the resulting distribution has no expectation).}
  \item{m0}{The default value \code{"ok"} indicates that ordinary
    kriging will be performed. Other options are \code{"kt"} for kriging
    with a trend model (universal kriging) and \code{"kte"} for kriging
    with external trend (covariates). If a numeric value is provided, this
    value is assumed to be the know mean and simple kriging is
    performed.
    If \code{"av"} the arithmetic mean of the data is assumed to be the
    know mean for simple kriging algorithm.}
  \item{nwin}{If \code{"full"}, all data values are used in the
    prediction at each prediction locations. An integer number specifies
    the number of closest neighbors be used for the prediction.}
  \item{n.samples.backtransform}{When transformations are used
    (specified by the argument \code{lambda}), back-transformations
    are usually performed by sampling the predictive distribution and
    back-transforming the sampled values. The exceptions are for
    \eqn{\lambda = 0}{lambda = 0} (log-transformation)  and
    \eqn{\lambda = 1}{lambda = 1} (no transformation). 
    This options defines how many samples are taken to perform the
    back-transformation.}
  \item{trend}{Required only if \code{m0="kt"} (universal kriging).
    Possible values are 1 or 2 corresponding to a first or second degree
    polynomial trend on the coordinates respectively.}
  \item{d}{Spatial dimension, \code{1} defines a prediction on a line, 2 on
    a plane (the default).}
  \item{ktedata}{Required only if \code{m0="kte"}. Values of the
    external trend (covariates) at the data locations.}
  \item{ktelocations}{Required only if \code{m0="kte"}. Values of the
    external trend (covariates) at the prediction locations.}
  \item{aniso.pars}{fixed parameters for geometric anisotropy
    correction. If \code{aniso.pars = F} no correction is done otherwise
    a vector with 2 values, anisotropy angle and ratio of ranges,
    must be provided. Anisotropy correction consists of a
    transformation in the data and prediction coordinates performed
    by the function \code{\link{coords.aniso}}.}
  \item{signal}{if \code{TRUE} the signal is predict, otherwise the
    variable is predict. If no transformation is performed the
    expectations are the same in both cases and the difference is only in
    the kriging variance, if nugget is different form zero.}
  \item{dist.epsilon}{a value defining a distance such that points
    which are closed that this are considered co-located.}
  \item{messages.screen}{a flag \code{TRUE} or \code{FALSE} indicating
    whether or not messages are printed on the screen (or output file)
    while the function is running.}
}

%\details{
%}

\value{
  A list with components:
  \item{predict}{predicted values}
  \item{krige.var}{kriging variances}
  \item{dif}{the difference between the predicted value and the global
    mean. Represents the contribution to the neighboring data to
    the prediction at each point.}
  \item{summary}{values of the arithmetic and weighted mean of the data and standard
    deviations. The weighted mean corresponds to the estimated value of
    the global mean. }
  \item{ktrend}{stores the matrix with trend if \code{mo="kt"}
    (universal kriging).}
  \item{ktetrend}{stores the matrix with trend if \code{mo="kt"}
    (universal kriging).}
  \item{beta}{the value of the mean which is implicitly estimated for
    \code{m0="ok", "kte"} or \code{"kt"}.}
  \item{wofmean}{weight of mean. The prediction can be seen as a
    weighted  average between the global mean and the the neighbors.
    The recorded value is the weight given to the mean.}
  \item{locations}{store the prediction locations.}
  \item{message}{status messages returned by the algorithm.}
  \item{call}{the function call.}
}

\references{
  Further information about \code{geoR/geoS} can be found at:\cr
  \url{http://www.maths.lancs.ac.uk/~ribeiro/geoR.html}
}

\note{
  This is a preliminary and inefficient function.
  For predictions using global neighborhood the function
  \code{\link{krige.conv}} should be used instead.
}

\author{
  Paulo J. Ribeiro Jr. \email{paulojus@est.ufpr.br} \cr
  Peter J. Diggle \email{p.diggle@lancaster.ac.uk}.
}

\seealso{\code{\link{krige.conv}}, \code{\link{krige.bayes}}.}


\examples{

if(is.R()) data(s100)
loci <- expand.grid(seq(0,1,l=31), seq(0,1,l=31))
kc <- ksline(s100, loc=loci, cov.pars=c(1, .25))
par(mfrow=c(1,2))
image.kriging(kc, loc=loci, main="kriging estimates")
image.kriging(kc, loc=loci, val=sqrt(kc$krige.var),
              main="kriging std. errors")

}

\keyword{spatial}%-- one or more ...


